@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import abc
+import inspect
 import typing
 
 from sqlalchemy import orm
@@ -20,9 +21,11 @@ E = typing.TypeVar("E")
 
 class Resource(abc.ABC, typing.Generic[T]):
     @classmethod
-    @abc.abstractmethod
     def resource_name(cls) -> str:
-        raise NotImplementedError
+        if inspect.isabstract(cls):
+            return cls.__name__
+        else:
+            return _get_next_descendant_of(cls, ancestor=Resource).__name__
 
     @abc.abstractmethod
     def rollback(self) -> None:
@@ -77,9 +80,10 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
         raise NotImplementedError
 
 
-class SqlAlchemyRepository(Repository[E], abc.ABC, typing.Generic[E]):
+class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
     def __init__(self, session: orm.Session):
         self._session = session
+        self._entity_type: typing.Optional[typing.Type[E]] = None
 
     def add(self, item: E, /) -> E:
         self.session.add(item)
@@ -96,17 +100,10 @@ class SqlAlchemyRepository(Repository[E], abc.ABC, typing.Generic[E]):
         return self.session.query(self.entity_type).get(item_id)
 
     @property
-    @abc.abstractmethod
     def entity_type(self) -> typing.Type[E]:
-        raise NotImplementedError
-
-    @classmethod
-    def resource_name(cls) -> str:
-        descendant = _get_next_descendant_of(cls=cls, ancestor=SqlAlchemyRepository)
-        if descendant is SqlAlchemyRepository:
-            return cls.__name__
-        else:
-            return descendant.__name__
+        if self._entity_type is None:
+            self._entity_type = typing.get_args(self.__class__)[0]
+        return self._entity_type
 
     def rollback(self) -> None:
         self.session.rollback()
